//
//  DraftsManager.swift
//  Tusker
//
//  Created by Shadowfacts on 10/22/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import Foundation

class DraftsManager: Codable {

    private(set) static var shared: DraftsManager = load()
    
    private static var documentsDirectory = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first!
    private static var archiveURL = DraftsManager.documentsDirectory.appendingPathComponent("drafts").appendingPathExtension("plist")
    
    static func save() {
        let encoder = PropertyListEncoder()
        let data = try? encoder.encode(shared)
        try? data?.write(to: archiveURL, options: .noFileProtection)
    }
    
    static func load() -> DraftsManager {
        let decoder = PropertyListDecoder()
        if let data = try? Data(contentsOf: archiveURL),
            let draftsManager = try? decoder.decode(DraftsManager.self, from: data) {
            return draftsManager
        }
        return DraftsManager()
    }
    
    private init() {}
    
    var drafts: [Draft] = []
    var sorted: [Draft] {
        return drafts.sorted(by: { $0.lastModified > $1.lastModified })
    }
    
    func create(text: String) {
        drafts.append(Draft(text: text, lastModified: Date()))
    }
    
    func remove(_ draft: Draft) {
        let index = drafts.firstIndex(of: draft)!
        drafts.remove(at: index)
    }
    
}

extension DraftsManager {
    class Draft: Codable, Equatable {
        let id: UUID
        private(set) var text: String
        private(set) var lastModified: Date
        
        init(text: String, lastModified: Date) {
            self.id = UUID()
            self.text = text
            self.lastModified = lastModified
        }
        
        func update(text: String) {
            self.text = text
            self.lastModified = Date()
        }
        
        static func ==(lhs: Draft, rhs: Draft) -> Bool {
            return lhs.id == rhs.id
        }
    }
}
