//
//  LargeImageExpandAnimationController.swift
//  Tusker
//
//  Created by Shadowfacts on 9/1/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import UIKit
import Gifu

class LargeImageExpandAnimationController: NSObject, UIViewControllerAnimatedTransitioning {
    
    func transitionDuration(using transitionContext: UIViewControllerContextTransitioning?) -> TimeInterval {
        return 0.2
    }
    
    func animateTransition(using transitionContext: UIViewControllerContextTransitioning) {
        guard let fromVC = transitionContext.viewController(forKey: .from),
            let toVC = transitionContext.viewController(forKey: .to) as? LargeImageViewController,
            let originFrame = toVC.originFrame else {
                return
        }
        
        let containerView = transitionContext.containerView
        let finalVCFrame = transitionContext.finalFrame(for: toVC)
        let image = toVC.imageView.image!
        let ratio = image.size.width / image.size.height
        let width = finalVCFrame.width
        let height = width / ratio
        let finalFrame = CGRect(x: finalVCFrame.midX - width / 2, y: finalVCFrame.midY - height / 2, width: width, height: height)
        
        let imageView = GIFImageView(frame: originFrame)
        imageView.image = toVC.imageView.image!
        if let gifData = toVC.gifData {
            imageView.animate(withGIFData: gifData)
        }
        imageView.contentMode = .scaleAspectFill
        imageView.layer.cornerRadius = toVC.originCornerRadius!
        imageView.layer.masksToBounds = true
        
        let blackView = UIView(frame: finalVCFrame)
        blackView.backgroundColor = .black
        blackView.alpha = 0
        
        containerView.addSubview(toVC.view)
        containerView.addSubview(blackView)
        containerView.addSubview(imageView)
        
        toVC.view.isHidden = true
        
        let duration = transitionDuration(using: transitionContext)
        UIView.animate(withDuration: duration, animations: {
            imageView.frame = finalFrame
            imageView.layer.cornerRadius = 0
            blackView.alpha = 1
        }, completion: { _ in
            // This shouldn't be necessary. I believe it's a workaround for using a XIB
            // for the large image VC. Without this, the final frame of the large image VC
            // is not set to the propper rect (it uses the frame of the preview device
            // in the XIB). When using a storyboard, the final frame is automatically set
            // (or UIKit does layout differently when loading the view) and this is not necessary.
            toVC.view.frame = finalVCFrame
            
            toVC.view.isHidden = false
            fromVC.view.isHidden = false
            blackView.removeFromSuperview()
            imageView.removeFromSuperview()
            transitionContext.completeTransition(!transitionContext.transitionWasCancelled)
        })
    }
    
}
